@@ -47,6 +47,7 @@ from task_processing.plugins.kubernetes.utils import (
 )
 from task_processing.plugins.kubernetes.utils import get_pod_volumes
 from task_processing.plugins.kubernetes.utils import get_sanitised_kubernetes_name
+from task_processing.plugins.kubernetes.utils import get_topology_spread_constraints
 
 
 logger = logging.getLogger(__name__)
@@ -60,6 +61,12 @@ SUPPORTED_POD_MODIFIED_EVENT_PHASES = {
     "Succeeded",
     "Unknown",
 }
+# arbitrarily chosen - we may honestly want to consider failing loudly if we actually get to
+# this amount of backoff since something is likely very wrong (this would require ~10 retries)
+MAX_WATCH_BACKOFF_S = 30
+# arbitrarily chosen - 1.5 seemed like a good compromise between multiple retries and giving the
+# control plane some breathing room
+RETRY_BACKOFF_EXPONENT = 1.5
 
 
 class KubernetesPodExecutor(TaskExecutor):
@@ -161,15 +168,18 @@ class KubernetesPodExecutor(TaskExecutor):
     def _pod_event_watch_loop(
         self, kube_client: KubeClient, watch: kube_watch.Watch
     ) -> None:
+        # this will generally only be used for recovering from expired resourceVersions exceptions
+        # we've seen the restart get stuck in a loop - even when we restart the watch anew - so
+        # let's add a small backoff time to avoid hammering the API server and hopefully avoid this
+        # since we're not sure if this restart loop is further hampering recovery
+        retry_attempt = 1
         logger.debug(f"Starting watching Pod events for namespace={self.namespace}.")
-        # TODO(TASKPROC-243): we'll need to correctly handle resourceVersion expiration for the case
-        # where the gap between task_proc shutting down and coming back up is long enough for data
-        # to have expired from etcd as well as actually restarting from the last resourceVersion in
-        # case of an exception
         # TODO: Do LIST + WATCH if we're not starting from a known/good resourceVersion to
         # guarantee that we always get ordered events (starting from a resourceVersion of 0)
         # has no such guarantees that the initial events will be ordered in any meaningful way
         # see: https://github.com/kubernetes/kubernetes/issues/74022
+        # NOTE: we'll also probably want to add some sort of reconciliation if we do ^
+        # since it's possible that we'll have skipped some events
         while not self.stopping:
             try:
                 for pod_event in watch.stream(
@@ -181,14 +191,40 @@ class KubernetesPodExecutor(TaskExecutor):
                     if not self.stopping:
                         logger.debug("Adding Pod event to pending event queue.")
                         self.pending_events.put(pod_event)
+
+                        # we reset the retry count unconditionally since this is simpler
+                        # than checking if we had any exponential backoff
+                        retry_attempt = 1
                     else:
                         break
             except ApiException as e:
                 if not self.stopping:
                     if not kube_client.maybe_reload_on_exception(exception=e):
+                        # we reset the resource version since it's possible that the last one is no
+                        # longer able to be resumed from
+                        # NOTE: this means that when we restart the stream, we'll get events for
+                        # every pod in the namespace - so we *shouldn't* miss any events at the
+                        # cost of likely re-processing events we've already handled (which should
+                        # be fine)
                         logger.exception(
                             "Unhandled API exception while watching Pod events - restarting watch!"
                         )
+                        watch.resource_version = None
+                        # this should be safe since this function runs in its own thread so this
+                        # sleep shouldn't block the workload using this plugin or the other
+                        # task_proc threads - additionally, we don't take any locks in this thread
+                        # so we shouldn't have to worry about any sort of deadlocks :)
+                        backoff_time = min(
+                            retry_attempt * RETRY_BACKOFF_EXPONENT,
+                            MAX_WATCH_BACKOFF_S,
+                        )
+                        logger.info(
+                            "Sleeping for %d seconds on attempt %d before retrying...",
+                            backoff_time,
+                            retry_attempt,
+                        )
+                        time.sleep(backoff_time)
+
             except Exception:
                 # we want to avoid a potentially misleading log message should we encounter
                 # an exception when we want to shutdown this thread since nothing of value
@@ -561,6 +597,9 @@ class KubernetesPodExecutor(TaskExecutor):
                     affinity=V1Affinity(
                         node_affinity=get_node_affinity(task_config.node_affinities),
                     ),
+                    topology_spread_constraints=get_topology_spread_constraints(
+                        task_config.topology_spread_constraints
+                    ),
                     # we're hardcoding this as Default as this is what we generally use
                     # internally - until we have a usecase for something that needs one
                     # of the other DNS policies, we can probably punt on plumbing all the
